@@ -1,44 +1,13 @@
-
-/**********************************************************************
- * GIFPlot 0.0
+/* ----------------------------------------------------------------------------- 
+ * gifplot.h
+ *
+ *     Main header file for the GIFPlot library.
  * 
- * Dave Beazley
- * 
- * Department of Computer Science        Theoretical Division (T-11)        
- * University of Utah                    Los Alamos National Laboratory     
- * Salt Lake City, Utah 84112            Los Alamos, New Mexico  87545      
- * beazley@cs.utah.edu                   beazley@lanl.gov                   
+ * Author(s) : David Beazley (beazley@cs.uchicago.edu)
+ * Copyright (C) 1995-1996
  *
- * Copyright (c) 1996
- * The Regents of the University of California and the University of Utah
- * All Rights Reserved
- *
- * Permission is hereby granted, without written agreement and without
- * license or royalty fees, to use, copy, modify, and distribute this
- * software and its documentation for any purpose, provided that 
- * (1) The above copyright notice and the following two paragraphs
- * appear in all copies of the source code and (2) redistributions
- * including binaries reproduces these notices in the supporting
- * documentation.   Substantial modifications to this software may be
- * copyrighted by their authors and need not follow the licensing terms
- * described here, provided that the new terms are clearly indicated in
- * all files where they apply.
- * 
- * IN NO EVENT SHALL THE AUTHOR, THE UNIVERSITY OF CALIFORNIA, THE 
- * UNIVERSITY OF UTAH OR DISTRIBUTORS OF THIS SOFTWARE BE LIABLE TO ANY
- * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
- * DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
- * EVEN IF THE AUTHORS OR ANY OF THE ABOVE PARTIES HAVE BEEN ADVISED OF
- * THE POSSIBILITY OF SUCH DAMAGE.
- *
- * THE AUTHOR, THE UNIVERSITY OF CALIFORNIA, AND THE UNIVERSITY OF UTAH
- * SPECIFICALLY DISCLAIM ANY WARRANTIES,INCLUDING, BUT NOT LIMITED TO, 
- * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND 
- * THE AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE,
- * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
- *
- **************************************************************************/
+ * See the file LICENSE for information on usage and redistribution.	
+ * ----------------------------------------------------------------------------- */
 
 #include <stdio.h>
 #include <fcntl.h>
@@ -47,7 +16,6 @@
 #include <string.h>
 #include <limits.h>
 
-
 #ifndef GIFPLOT_H
 
 /* Pixel is 8-bits */
@@ -63,7 +31,7 @@ typedef float Zvalue;
 
 typedef struct ColorMap {
   unsigned char          *cmap;
-  char          *name;
+  char                   *name;
 } ColorMap;
 
 extern ColorMap *new_ColorMap(char *filename);
@@ -102,7 +70,6 @@ typedef struct FrameBuffer {
   int             ymax;
 } FrameBuffer;
 
-/* #define ZMIN     FLT_MAX   */ /* Minimum Z coordinate */
 #define ZMIN  1e+36
 
 /* FrameBuffer Methods */
@@ -112,47 +79,30 @@ extern void         delete_FrameBuffer(FrameBuffer *frame);
 extern int          FrameBuffer_resize(FrameBuffer *frame, int width, int height);
 extern void         FrameBuffer_clear(FrameBuffer *frame, Pixel color);
 extern void         FrameBuffer_plot(FrameBuffer *frame, int x, int y, Pixel color);
-extern void         FrameBuffer_horizontal(FrameBuffer *frame, int xmin,
-					   int xmax, int y, Pixel color);
-extern void         FrameBuffer_horizontalInterp(FrameBuffer *f, int xmin,
-						 int xmax, int y,  Pixel c1, Pixel c2);
-extern void         FrameBuffer_vertical(FrameBuffer *frame, int ymin, int ymax,
-					 int x, Pixel color);
-
-extern void         FrameBuffer_box(FrameBuffer *frame, int x1, int y1,
-				    int x2, int y2, Pixel color);
-extern void         FrameBuffer_solidbox(FrameBuffer *frame, int x1, int y1,
-					 int x2, int y2, Pixel color);
-extern void         FrameBuffer_interpbox(FrameBuffer *f, int x1, int y1, int x2, int y2,
-					  Pixel c1, Pixel c2, Pixel c3, Pixel c4);
-
-extern void         FrameBuffer_circle(FrameBuffer *frame, int x1, int y1,
-				       int radius, Pixel color);
-extern void         FrameBuffer_solidcircle(FrameBuffer *frame, int x1, int y1,
-					    int radius, Pixel color);
-extern void         FrameBuffer_line(FrameBuffer *frame, int x1, int y1,
-				     int x2, int y2, Pixel color);
-extern void         FrameBuffer_setclip(FrameBuffer *frame, int xmin, int ymin,
-					int xmax, int ymax);
+extern void         FrameBuffer_horizontal(FrameBuffer *frame, int xmin, int xmax, int y, Pixel color);
+extern void         FrameBuffer_horizontalinterp(FrameBuffer *f, int xmin, int xmax, int y,  Pixel c1, Pixel c2);
+extern void         FrameBuffer_vertical(FrameBuffer *frame, int ymin, int ymax, int x, Pixel color);
+extern void         FrameBuffer_box(FrameBuffer *frame, int x1, int y1, int x2, int y2, Pixel color);
+extern void         FrameBuffer_solidbox(FrameBuffer *frame, int x1, int y1, int x2, int y2, Pixel color);
+extern void         FrameBuffer_interpbox(FrameBuffer *f, int x1, int y1, int x2, int y2, Pixel c1, Pixel c2, Pixel c3, Pixel c4);
+extern void         FrameBuffer_circle(FrameBuffer *frame, int x1, int y1, int radius, Pixel color);
+extern void         FrameBuffer_solidcircle(FrameBuffer *frame, int x1, int y1, int radius, Pixel color);
+extern void         FrameBuffer_line(FrameBuffer *frame, int x1, int y1, int x2, int y2, Pixel color);
+extern void         FrameBuffer_setclip(FrameBuffer *frame, int xmin, int ymin, int xmax, int ymax);
 extern void         FrameBuffer_noclip(FrameBuffer *frame);
-extern int          FrameBuffer_makeGIF(FrameBuffer *frame, ColorMap *cmap,
-					void *buffer, unsigned int maxsize);
-extern int FrameBuffer_writeGIF(FrameBuffer *f, ColorMap *c, char *filename);
+extern int          FrameBuffer_makeGIF(FrameBuffer *frame, ColorMap *cmap, void *buffer, unsigned int maxsize);
+extern int          FrameBuffer_writeGIF(FrameBuffer *f, ColorMap *c, char *filename);
 extern void         FrameBuffer_zresize(FrameBuffer *f, int width, int height);
 extern void         FrameBuffer_zclear(FrameBuffer *f);
-
 extern void         FrameBuffer_solidtriangle(FrameBuffer *f, int x1, int y1, int x2, int y2, int x3, int y3, Pixel c);
-
-void FrameBuffer_interptriangle(FrameBuffer *f, int tx1, int ty1, Pixel c1,
-				int tx2, int ty2, Pixel c2,
-				int tx3, int ty3, Pixel c3);
-
+extern void         FrameBuffer_interptriangle(FrameBuffer *f, int tx1, int ty1, Pixel c1,
+                                               int tx2, int ty2, Pixel c2, int tx3, int ty3, Pixel c3);
 
 #define  HORIZONTAL  1
 #define  VERTICAL    2
 
-extern void FrameBuffer_drawchar(FrameBuffer *frame, int x, int y, int fgcolor, int bgcolor, char chr, int orientation);
-extern void FrameBuffer_drawstring(FrameBuffer *f, int x, int y, int fgcolor, int bgcolor, char *text, int orientation);
+extern void         FrameBuffer_drawchar(FrameBuffer *frame, int x, int y, int fgcolor, int bgcolor, char chr, int orientation);
+extern void         FrameBuffer_drawstring(FrameBuffer *f, int x, int y, int fgcolor, int bgcolor, char *text, int orientation);
 
 /* ------------------------------------------------------------------------
    PixMap
@@ -175,9 +125,9 @@ extern void      delete_PixMap(PixMap *pm);
 extern void      PixMap_set(PixMap *pm, int x, int y, int pix);
 extern void      FrameBuffer_drawpixmap(FrameBuffer *f, PixMap *pm, int x, int y, int fgcolor, int bgcolor);
 
-#define   TRANSPARENT  0
-#define   FOREGROUND   1
-#define   BACKGROUND   2
+#define   GIFPLOT_TRANSPARENT  0
+#define   GIFPLOT_FOREGROUND   1
+#define   GIFPLOT_BACKGROUND   2
 
 /* ------------------------------------------------------------------------
    Plot2D
@@ -203,40 +153,29 @@ typedef struct Plot2D {
 
 /* 2D Plot methods */
 
-extern Plot2D  *new_Plot2D(FrameBuffer *frame,double xmin,double ymin,
-			   double xmax, double ymax);
+extern Plot2D  *new_Plot2D(FrameBuffer *frame,double xmin,double ymin, double xmax, double ymax);
 extern void     delete_Plot2D(Plot2D *p2);
 extern Plot2D  *Plot2D_copy(Plot2D *p2);
 extern void     Plot2D_clear(Plot2D *p2, Pixel c);
 extern void     Plot2D_setview(Plot2D *p2, int vxmin, int vymin, int vxmax, int vymax);
-extern void     Plot2D_setrange(Plot2D *p2, double xmin, double ymin,
-				double xmax, double ymax);
+extern void     Plot2D_setrange(Plot2D *p2, double xmin, double ymin, double xmax, double ymax);
 extern void     Plot2D_setscale(Plot2D *p2, int xscale, int yscale);
 extern void     Plot2D_plot(Plot2D *p2, double x, double y, Pixel color);
-extern void     Plot2D_box(Plot2D *p2, double x1, double y1,
-			   double x2, double y2, Pixel color);
-extern void     Plot2D_solidbox(Plot2D *p2, double x1, double y1,
-				double x2, double y2, Pixel color);
-extern void     Plot2D_interpbox(Plot2D *p2, double x1, double y1, double x2,
-				 double y2, Pixel c1, Pixel c2, Pixel c3, Pixel c4);
-
-extern void     Plot2D_circle(Plot2D *p2, double x, double y,
-			      double radius, Pixel color);
-extern void     Plot2D_solidcircle(Plot2D *p2, double x, double y,
-				   double radius, Pixel color);
-extern void     Plot2D_line(Plot2D *p2, double x1, double y1,
-			    double x2, double y2, Pixel color);
+extern void     Plot2D_box(Plot2D *p2, double x1, double y1,  double x2, double y2, Pixel color);
+extern void     Plot2D_solidbox(Plot2D *p2, double x1, double y1,double x2, double y2, Pixel color);
+extern void     Plot2D_interpbox(Plot2D *p2, double x1, double y1, double x2, double y2, Pixel c1, Pixel c2, Pixel c3, Pixel c4);
+extern void     Plot2D_circle(Plot2D *p2, double x, double y, double radius, Pixel color);
+extern void     Plot2D_solidcircle(Plot2D *p2, double x, double y, double radius, Pixel color);
+extern void     Plot2D_line(Plot2D *p2, double x1, double y1, double x2, double y2, Pixel color);
 extern void     Plot2D_start(Plot2D *p2);
 extern void     Plot2D_drawpixmap(Plot2D *p2, PixMap *pm, double x, double y, Pixel color, Pixel bgcolor);
 extern void     Plot2D_xaxis(Plot2D *p2, double x, double y, double xtick, int ticklength, Pixel c);
 extern void     Plot2D_yaxis(Plot2D *p2, double x, double y, double ytick, int ticklength, Pixel c);
 extern void     Plot2D_triangle(Plot2D *p2, double x1, double y1, double x2, double y2, double x3, double y3, Pixel c);
-
 extern void     Plot2D_solidtriangle(Plot2D *p2, double x1, double y1, double x2, double y2, double x3, double y3, Pixel c);
-
-extern void Plot2D_interptriangle(Plot2D *p2, double x1, double y1, Pixel c1,
-			   double x2, double y2, Pixel c2,
-			   double x3, double y3, Pixel c3);
+extern void     Plot2D_interptriangle(Plot2D *p2, double x1, double y1, Pixel c1,
+                                                  double x2, double y2, Pixel c2,
+                                                  double x3, double y3, Pixel c3);
 
 #define LINEAR     10
 #define LOG        11
