@@ -2,9 +2,9 @@
 /* SWIG version information */
 
 #ifndef SWIG_VERSION
-#define SWIG_VERSION "1.3.10u-20011129-2251"
+#define SWIG_VERSION "1.3u-20010628-1103"
 #endif
 
 #ifndef SWIG_SPIN
-#define SWIG_SPIN "10"
+#define SWIG_SPIN "(Alpha 5)"
 #endif
