@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17.2.30 2002/01/27 04:02:51 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17 2001/06/19 13:41:49 mkoeppe Exp $";
 
 /***********************************************************************
- * $Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17.2.30 2002/01/27 04:02:51 beazley Exp $
+ * $Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17 2001/06/19 13:41:49 mkoeppe Exp $
  *
  * guile.cxx
  *
@@ -25,35 +25,30 @@ static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx
 
 #include "mod11.h"
 #include "guile.h"
-#ifndef MACSWIG
 #include "swigconfig.h"
-#endif
-#include <ctype.h>
 
 static char *guile_usage = (char*)"\
 Guile Options (available with -guile)\n\
      -ldflags        - Print runtime libraries to link with\n\
+     -module name    - Set name of module [default \"swig\"]\n\
      -prefix name    - Use NAME as prefix [default \"gswig_\"]\n\
      -package name   - Set the path of the module [default NULL]\n\
      -linkage lstyle - Use linkage protocol LSTYLE [default `module']\n\
      -procdoc file   - Output procedure documentation to FILE\n\
+\n\
      -procdocformat format - Output procedure documentation in FORMAT;\n\
                              one of `guile-1.4', `plain', `texinfo'\n\
-     -scmstub file   - Output Scheme FILE with module declaration and\n\
-                       exports; only with `passive' and `simple' linkage\n\
+  The module option does not create a guile module with a separate name\n\
+  space.  It specifies the name of the initialization function and is\n\
+  called a module here so that it is compadible with the rest of SWIG.\n\
 \n\
   When unspecified, the default LSTYLE is `simple'.  For native Guile\n\
   module linking (for Guile versions >=1.5.0), use `module'.  Other\n\
-  LSTYLE values are: `passive' for passive linking (no C-level\n\
-  module-handling  code), `ltdlmod' for Guile's old dynamic module\n\
-  convention (versions <= 1.4), or `hobbit' for hobbit modules.\n\
+  LSTYLE values are: `passive' for passive linking (no module-handling\n\
+  code), `ltdlmod' for Guile's old dynamic module convention\n\
+  (versions <= 1.4), or `hobbit' for hobbit modules.\n\
 \n";
 
-static  File         *f_runtime = 0;
-static  File         *f_header = 0;
-static  File         *f_wrappers = 0;
-static  File         *f_init = 0;
-
 // ---------------------------------------------------------------------
 // GUILE ()
 // ---------------------------------------------------------------------
@@ -62,7 +57,7 @@ GUILE::GUILE ()
 {
   // Set global vars
 
-  SWIG_typemap_lang("guile");
+  typemap_lang = (char*)"guile";
 
   // Set class vars
 
@@ -71,28 +66,25 @@ GUILE::GUILE ()
   package = NULL;
   linkage = GUILE_LSTYLE_SIMPLE;
   procdoc = NULL;
-  scmstub = NULL;
   docformat = GUILE_1_4;
   emit_setters = 0;
   struct_member = 0;
   before_return = NULL;
   exported_symbols = NewString("");
-  scmtext = NewString("");
-  Swig_register_filebyname("scheme", scmtext);
 }
 
 // ---------------------------------------------------------------------
-// GUILE::main()
+// GUILE::parse_args(int argc, char *argv[])
 //
 // Parse arguments.
 // ---------------------------------------------------------------------
 
 void
-GUILE::main (int argc, char *argv[])
+GUILE::parse_args (int argc, char *argv[])
 {
   int i, orig_len;
 
-  SWIG_library_directory("guile");
+  sprintf (LibDir, "%s", "guile");
 
   // Look for certain command line options
   for (i = 1; i < argc; i++) {
@@ -123,10 +115,21 @@ GUILE::main (int argc, char *argv[])
 	  Swig_arg_error();
 	}
       }
+      else if (strcmp (argv[i], "-module") == 0) {
+	if (argv[i + 1]) {
+	  set_module (argv[i + 1]);
+	  Swig_mark_arg (i);
+	  Swig_mark_arg (i + 1);
+	  ++i;
+	} else {
+	  Swig_arg_error();
+	}
+      }
       else if (strcmp (argv[i], "-ldflags") == 0) {
 	printf("%s\n", SWIG_GUILE_RUNTIME);
 	SWIG_exit (EXIT_SUCCESS);
       }
+      /* The upcase variant is a historic artefact. */
       else if (strcmp (argv[i], "-Linkage") == 0
 	       || strcmp (argv[i], "-linkage") == 0) {
         if (argv[i + 1]) {
@@ -151,7 +154,7 @@ GUILE::main (int argc, char *argv[])
       }
       else if (strcmp (argv[i], "-procdoc") == 0) {
 	if (argv[i + 1]) {
-	  procdoc = NewFile(argv[i + 1], (char *) "w");
+	  procdoc = NewFile(argv[i + 1], "w");
 	  Swig_mark_arg (i);
           Swig_mark_arg (i + 1);
 	  i++;
@@ -175,16 +178,6 @@ GUILE::main (int argc, char *argv[])
 	emit_setters = 1;
 	Swig_mark_arg (i);
       }
-      else if (strcmp (argv[i], "-scmstub") == 0) {
-	if (argv[i + 1]) {
-	  scmstub = NewFile(argv[i + 1], (char *) "w");
-	  Swig_mark_arg (i);
-          Swig_mark_arg (i + 1);
-	  i++;
-	} else {
-	  Swig_arg_error();
-        }
-      }
     }
   }
 
@@ -204,48 +197,90 @@ GUILE::main (int argc, char *argv[])
 }
 
 // --------------------------------------------------------------------
-// GUILE::top()
+// GUILE::parse()
+//
+// Parse the input file
 // --------------------------------------------------------------------
 
-int
-GUILE::top(Node *n)
+void
+GUILE::parse ()
 {
+  // Print out GUILE specific headers
 
-  /* Initialize all of the output files */
-  String *outfile = Getattr(n,"outfile");
+  headers();
 
-  f_runtime = NewFile(outfile,"w");
-  if (!f_runtime) {
-    Printf(stderr,"*** Can't open '%s'\n", outfile);
-    SWIG_exit(EXIT_FAILURE);
-  }
-  f_init = NewString("");
-  f_header = NewString("");
-  f_wrappers = NewString("");
+  // Run the parser
 
-  /* Register file targets with the SWIG file handler */
-  Swig_register_filebyname("header",f_header);
-  Swig_register_filebyname("wrapper",f_wrappers);
-  Swig_register_filebyname("runtime",f_runtime);
-  Swig_register_filebyname("init",f_init);
+  yyparse();
+}
 
+// ---------------------------------------------------------------------
+// GUILE::set_module(char *mod_name)
+//
+// Sets the module name.
+// Does nothing if it's already set (so it can be overridden as a command
+// line option).
+//
+//----------------------------------------------------------------------
+
+void
+GUILE::set_module (char *mod_name)
+{
+  if (module) return;
+
+  module = new char [strlen (mod_name) + 1];
+  strcpy (module, mod_name);
+}
+
+// ---------------------------------------------------------------------
+// GUILE::set_init(char *iname)
+//
+// Sets the initialization function name.
+// Does nothing if it's already set
+//
+//----------------------------------------------------------------------
+
+void
+GUILE::set_init (char *iname)
+{
+  abort ();                             // for now -ttn
+  set_module (iname);
+}
+
+// ---------------------------------------------------------------------
+// GUILE::headers(void)
+//
+// Generate the appropriate header files for GUILE interface.
+// ----------------------------------------------------------------------
+
+void
+GUILE::headers (void)
+{
   Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
   Swig_banner (f_runtime);
 
   Printf (f_runtime, "/* Implementation : GUILE */\n\n");
 
-  /* Write out directives and declarations */
+  // Write out directives and declarations
 
   if (NoInclude) {
     Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
   }
-  
-  module = Swig_copy_string(Char(Getattr(n,"name")));
+}
 
+// --------------------------------------------------------------------
+// GUILE::initialize()
+//
+// Output initialization code that registers functions with the
+// interface.
+// ---------------------------------------------------------------------
+
+void
+GUILE::initialize (void)
+{
   if (CPlusPlus) {
     Printf(f_runtime, "extern \"C\" {\n\n");
   }
-
   switch (linkage) {
   case GUILE_LSTYLE_SIMPLE:
     /* Simple linkage; we have to export the SWIG_init function. The user can
@@ -263,50 +298,6 @@ GUILE::top(Node *n)
   if (CPlusPlus) {
     Printf(f_runtime, "\n}\n");
   }
-
-  Language::top(n);
-  
-  /* Close module */
-
-  Printf(f_wrappers,"#ifdef __cplusplus\nextern \"C\" {\n#endif\n");
-
-  SwigType_emit_type_table (f_runtime, f_wrappers);
-
-  Printf(f_wrappers,"#ifdef __cplusplus\n}\n#endif\n");
-
-  Printf (f_init, "SWIG_Guile_RegisterTypes(swig_types, swig_types_initial);\n");
-  Printf (f_init, "}\n\n");
-  char module_name[256];
-
-  if (!module)
-    sprintf(module_name, "swig");
-  else {
-    if (package)
-      sprintf(module_name,"%s/%s", package,module);
-    else
-      strcpy(module_name,module);
-  }
-  emit_linkage (module_name);
-  
-  if (procdoc) {
-    Delete(procdoc);
-    procdoc = NULL;
-  }
-  if (scmstub) {
-    Delete(scmstub);
-    scmstub = NULL;
-  }
-
-  /* Close all of the files */
-  Dump(f_header,f_runtime);
-  Dump(f_wrappers,f_runtime);
-  Wrapper_pretty_print(f_init,f_runtime);
-  Delete(f_header);
-  Delete(f_wrappers);
-  Delete(f_init);
-  Close(f_runtime);
-  Delete(f_runtime);
-  return SWIG_OK;
 }
 
 void
@@ -372,7 +363,6 @@ GUILE::emit_linkage (char *module_name)
       Printf(f_init, "    SCM module = scm_c_define_module(\"%s\",\n", mod);
       Printf(f_init, "      SWIG_init_helper, NULL);\n");
       Printf(f_init, "    return SCM_UNSPECIFIED;\n");
-      Delete(mod);
     }
     Printf (f_init, "}\n");
     break;
@@ -394,36 +384,43 @@ GUILE::emit_linkage (char *module_name)
   default:
     abort();                            // for now
   }
-
-  if (scmstub) {
-    /* Emit Scheme stub if requested */
-    DOHString *mod = NewString(module_name);
-    Replace(mod, "/", " ", DOH_REPLACE_ANY);
-    Printf (scmstub, ";;; -*- buffer-read-only: t -*- vi: set ro: */\n");
-    Printf (scmstub, ";;; Automatically generated by SWIG; do not edit.\n\n");
-    if (linkage == GUILE_LSTYLE_SIMPLE
-	|| linkage == GUILE_LSTYLE_PASSIVE)
-      Printf (scmstub, "(define-module (%s))\n\n", mod);
-    Delete(mod);
-    Printf (scmstub, "%s", scmtext);
-    if ((linkage == GUILE_LSTYLE_SIMPLE
-	|| linkage == GUILE_LSTYLE_PASSIVE)
-	&& Len(exported_symbols) > 0) {
-      DOHString *ex = NewString(exported_symbols);
-      Replace(ex, ", ", "\n        ", DOH_REPLACE_ANY);
-      Replace(ex, "\"", "", DOH_REPLACE_ANY);
-      Chop(ex);
-      Printf(scmstub, "\n(export %s)\n", ex);
-      Delete(ex);
-    }
-  }
-  
   Delete(module_func);
   if (CPlusPlus) {
     Printf(f_init, "\n}\n");
   }
 }
 
+// ---------------------------------------------------------------------
+// GUILE::close(void)
+//
+// Wrap things up.  Close initialization function.
+// ---------------------------------------------------------------------
+
+void
+GUILE::close (void)
+{
+  SwigType_emit_type_table (f_runtime, f_wrappers);
+
+  Printf (f_init, "SWIG_Guile_RegisterTypes(swig_types, swig_types_initial);\n");
+  Printf (f_init, "}\n\n");
+  char module_name[256];
+
+  if (!module)
+    sprintf(module_name, "swig");
+  else {
+    if (package)
+      sprintf(module_name,"%s/%s", package,module);
+    else
+      strcpy(module_name,module);
+  }
+  emit_linkage (module_name);
+  
+  if (procdoc) {
+    Delete(procdoc);
+    procdoc = NULL;
+  }
+}
+
 /* Return true iff T is a pointer type */
 
 static int
@@ -432,6 +429,103 @@ is_a_pointer (SwigType *t)
   return SwigType_ispointer(SwigType_typedef_resolve_all(t));
 }
 
+/* Same as Swig_typemap_lookup but fall back to `int' when `enum' is
+   requested -- enum handling is somewhat broken in the 1.1 parser.
+   But we don't want to change it now since it is deprecated. */
+
+static char *
+guile_typemap_lookup(const char *op, SwigType *type, const String_or_char *pname, String_or_char *source,
+		     String_or_char *target, Wrapper *f)
+{
+  char *tm;
+  tm = Swig_typemap_lookup((char*) op, type, (char*)pname, source, target, f);
+  if (!tm) {
+    SwigType *base = SwigType_typedef_resolve_all(type);
+    if (strncmp(Char(base), "enum ", 5)==0)
+      tm = Swig_typemap_lookup((char*) op, (char*) "int", (char*)pname, source, target, f);
+  }
+  return tm;
+}
+
+/* Lookup a typemap, replace all relevant parameters and write it to
+   the given generalized file. Return 0 if no typemap found. */
+
+static int
+guile_do_typemap(DOHFile *file, const char *op,
+		 SwigType *type, const String_or_char *arg,
+		 String_or_char *source, String_or_char *target,
+		 int argnum, DOHString *name, Wrapper *f,
+		 int nonewline_p)
+{
+  char *tm;
+  if ((tm = guile_typemap_lookup(op, type, arg,
+				 source, target, f))) {
+    String *s = NewString(tm);
+    String *descriptor = NewString("");
+    String *basedescriptor = NewString("");
+    String *stardescriptor = NewString("");
+    char argnum_s[10];
+    SwigType *startype = NULL;
+
+    if (SwigType_ispointer(type)) {
+      startype = Copy(type);
+      SwigType_del_pointer(startype);
+      Printf(stardescriptor, "SWIGTYPE%s",
+	     SwigType_manglestr(startype));
+    }
+    else Printf(stardescriptor, "SWIGTYPE_BAD");
+    Printf(descriptor, "SWIGTYPE%s",
+	   SwigType_manglestr(type));
+    Printf(basedescriptor, "SWIGTYPE%s",
+	   SwigType_manglestr(SwigType_base(type)));
+    sprintf(argnum_s, "%d", argnum);
+    Replace(s,"$argnum", argnum_s, DOH_REPLACE_ANY);
+    Replace(s,"$arg",    arg,      DOH_REPLACE_ANY);
+    Replace(s,"$name",   name,     DOH_REPLACE_ANY);
+    if (Replace(s, "$descriptor",
+		descriptor, DOH_REPLACE_ANY))
+      SwigType_remember(type);
+    if (Replace(s, "$basedescriptor",
+		basedescriptor, DOH_REPLACE_ANY))
+      SwigType_remember(SwigType_base(type));
+    if (Replace(s, "$*descriptor", stardescriptor,
+		DOH_REPLACE_ANY)) {
+      if (!startype) {
+	Printf (stderr, "%s : Line %d. $*descriptor is meaningless for non-pointer types.\n",
+		input_file, line_number);
+	error_count++;
+      }
+      else SwigType_remember(startype);
+    }
+    if (nonewline_p)
+      Printv(file, s, 0);
+    else Printv(file, s, "\n", 0);
+    Delete(s);
+    return 1;
+  }
+  else return 0;
+}
+
+/* Lookup a documentation typemap, replace all relevant parameters and
+   write it to the given generalized file, providing a sensible
+   default value. */
+
+static void
+guile_do_doc_typemap(DOHFile *file, const char *op,
+		     SwigType *type, const String_or_char *arg,
+		     int argnum, DOHString *name, Wrapper *f)
+{
+  if (!guile_do_typemap(file, op, type, arg,
+			NULL, NULL, argnum, name, f, 1)) {
+    /* FIXME: Can't we provide this default via a typemap as well? */
+    String *s = NewString(SwigType_str(type, 0));
+    Chop(s);
+    if (arg) Printf(file, "(%s <%s>)", arg, s);
+    else Printf(file, "<%s>", s);
+    Delete(s);
+  }
+}
+
 /* Report an error handling the given type. */
 
 static void
@@ -471,15 +565,15 @@ GUILE::write_doc(const String *proc_name,
 
 
 // ----------------------------------------------------------------------
-// GUILE::functionWrapper()
+// GUILE::create_function(char *name, char *iname, SwigType *d,
+//                             ParmList *l)
+//
 // Create a function declaration and register it with the interpreter.
 // ----------------------------------------------------------------------
 
-int
-GUILE::functionWrapper(Node *n) {
-  char *iname = GetChar(n,"sym:name");
-  SwigType *d = Getattr(n,"type");
-  ParmList *l = Getattr(n,"parms");
+void
+GUILE::create_function (char *name, char *iname, SwigType *d, ParmList *l)
+{
   Parm *p;
   DOHString *proc_name = 0;
   char source[256], target[256], wname[256];
@@ -490,16 +584,13 @@ GUILE::functionWrapper(Node *n) {
   String *returns = NewString("");
   int returns_list = 0;
   String *tmp = NewString("");
-  String *tm;
   int i;
   int numargs = 0;
-  int numreq = 0;
+  int numopt = 0;
 
   // Make a wrapper name for this
-  {
-    String *n = Swig_name_mangle(iname);
-    strcpy(wname, Char(Swig_name_wrapper(n)));
-  }
+  strcpy(wname, Char(Swig_name_wrapper(name)));
+
   // Build the name for scheme.
   proc_name = NewString(iname);
   Replace(proc_name,"_", "-", DOH_REPLACE_ANY);
@@ -507,31 +598,14 @@ GUILE::functionWrapper(Node *n) {
   /* Emit locals etc. into f->code; figure out which args to ignore */
   emit_args (d, l, f);
 
-  /* Attach the standard typemaps */
-  emit_attach_parmmaps(l,f);
-
-  /* Get number of required and total arguments */
-  numargs = emit_num_arguments(l);
-  numreq  = emit_num_required(l);
-
   /* Declare return variable */
 
   Wrapper_add_local (f,"gswig_result", "SCM gswig_result");
   Wrapper_add_local (f,"gswig_list_p", "int gswig_list_p = 0");
 
-  /* Get the output typemap so we can start generating documentation.  Don't
-     worry, the returned string is saved as 'tmap:out' */
-
-  Swig_typemap_lookup_new("out",n,"result",0);
-  
-  if ((tm = Getattr(n,"tmap:out:doc"))) {
-    Printv(returns,tm,0);
-  } else {
-    String *s = SwigType_str(d,0);
-    Chop(s);
-    Printf(returns,"<%s>",s);
-    Delete(s);
-  }
+  if (procdoc)
+    guile_do_doc_typemap(returns, "outdoc", d, NULL,
+			 0, proc_name, f);
 
   /* Open prototype and signature */
 
@@ -540,100 +614,74 @@ GUILE::functionWrapper(Node *n) {
 
   /* Now write code to extract the parameters */
 
-  for (i = 0, p = l; i < numargs; i++) {
-    while (Getattr(p,"tmap:ignore")) {
-      p = Getattr(p,"tmap:ignore:next");
-    }
-
-    SwigType *pt = Getattr(p,"type");
-    String   *pn = Getattr(p,"name");
-    String   *ln = Getattr(p,"lname");
-    int opt_p = (i >= numreq);
+  for (p = l, i = 0; p; p=Getnext(p), i++) {
+    SwigType *pt = Gettype(p);
+    String   *pn = Getname(p);
+    int opt_p = (Getvalue(p)
+		 || Swig_typemap_search((char*)"default",pt,pn));
 
     // Produce names of source and target
     sprintf(source,"s_%d",i);
-    sprintf(target,"%s", Char(ln));
+    sprintf(target,"%s", Char(Getlname(p)));
 
-    if (i!=0) Printf(f->def,", ");
-    Printf(f->def,"SCM s_%d", i);
-    if (opt_p) {
-      Printf(f->code,"    if (s_%d != GH_NOT_PASSED) {\n", i);
-    }
-    if ((tm = Getattr(p,"tmap:in"))) {
-      Replace(tm,"$source",source,DOH_REPLACE_ANY); /* Deprecated */
-      Replace(tm,"$target",target,DOH_REPLACE_ANY); /* Deprecated */
-      Replace(tm,"$input",source,DOH_REPLACE_ANY);
-      Setattr(p,"emit:input", source);
-      Printv(f->code,tm,"\n",0);
+    // Handle parameter types.
 
+    if (Getignore(p))
+      Printv(f->code, "/* ", pn, " ignored... */\n", 0);
+    else {
+      if (numargs!=0) Printf(f->def,", ");
+      Printf(f->def,"SCM s_%d", i);
+      if (opt_p) {
+	numopt++;
+	Printf(f->code,"    if (s_%d != GH_NOT_PASSED) {\n", i);
+      }
+      ++numargs;
+      if (guile_do_typemap(f->code, "in", pt, pn,
+			   source, target, numargs, proc_name, f, 0)) {
+	/* nothing to do */
+      }
+      else {
+        throw_unhandled_guile_type_error (pt);
+      }
       if (procdoc) {
 	/* Add to signature */
 	Printf(signature, " ");
-	if ((tm = Getattr(p,"tmap:in:doc"))) {
-	  Replaceall(tm,"$name", pn);
-	  Printv(signature,tm,0);
-	} else {
-	  String *s = SwigType_str(pt,0);
-	  Chop(s);
-	  Printf(signature,"(%s <%s>)", pn, s);
-	  Delete(s);
+	guile_do_doc_typemap(signature, "indoc", pt, pn,
+			     numargs, proc_name, f);
+      }
+      if (opt_p)
+	Printf(f->code,"    }\n");
+    }
+
+    /* Check if there are any constraints. */
+
+    guile_do_typemap(f->code, "check", pt, pn,
+		     source, target, numargs, proc_name, f, 0);
+
+    /* Pass output arguments back to the caller. */
+
+    guile_do_typemap(outarg, "argout", pt, pn,
+		     source, target, numargs, proc_name, f, 0);
+
+    if (procdoc) {
+      /* Document output arguments */
+      Clear(tmp);
+      if (guile_do_typemap(tmp, "argoutdoc", pt, pn,
+			   source, target, numargs, proc_name, f, 1)) {
+	if (Len(returns) == 0) { /* unspecified -> singleton */
+	  Printv(returns, tmp, 0);
+	}
+	else { /* append to list */
+	  Printv(returns, " ", tmp, 0);
+	  returns_list = 1;
 	}
       }
-      p = Getattr(p,"tmap:in:next");
-    } else {
-      throw_unhandled_guile_type_error (pt);
-      p = nextSibling(p);
     }
-    if (opt_p)
-      Printf(f->code,"    }\n");
-  }
 
-  /* Insert constraint checking code */
-  for (p = l; p;) {
-    if ((tm = Getattr(p,"tmap:check"))) {
-      Replace(tm,"$target",Getattr(p,"lname"),DOH_REPLACE_ANY);
-      Printv(f->code,tm,"\n",0);
-      p = Getattr(p,"tmap:check:next");
-    } else {
-      p = nextSibling(p);
-    }
-  }
-  /* Pass output arguments back to the caller. */
+    // free up any memory allocated for the arguments.
 
-  /* Insert argument output code */
-  for (p = l; p;) {
-    if ((tm = Getattr(p,"tmap:argout"))) {
-      Replace(tm,"$source",Getattr(p,"lname"),DOH_REPLACE_ANY);   /* Deprecated */
-      Replace(tm,"$target",Getattr(p,"lname"),DOH_REPLACE_ANY);   /* Deprecated */
-      Replace(tm,"$arg",Getattr(p,"emit:input"), DOH_REPLACE_ANY);
-      Replace(tm,"$input",Getattr(p,"emit:input"), DOH_REPLACE_ANY);
-      Printv(outarg,tm,"\n",0);
-      if (procdoc) {
-	if ((tm = Getattr(p,"tmap:argout:doc"))) {
-	  Replaceall(tm,"$name",Getattr(p,"name"));
-	  if (Len(returns)) {
-	    Printv(returns," ", tm, 0);
-	    returns_list = 1;
-	  } else {
-	    Printv(returns,tm,0);
-	  }
-	}
-      }
-      p = Getattr(p,"tmap:argout:next");
-    } else {
-      p = nextSibling(p);
-    }
-  }
-
-  /* Insert cleanup code */
-  for (p = l; p;) {
-    if ((tm = Getattr(p,"tmap:freearg"))) {
-      Replace(tm,"$target",Getattr(p,"lname"),DOH_REPLACE_ANY);
-      Printv(cleanup,tm,"\n",0);
-      p = Getattr(p,"tmap:freearg:next");
-    } else {
-      p = nextSibling(p);
-    }
+    guile_do_typemap(cleanup, "freearg", pt, pn,
+		     source, target, numargs, proc_name, f, 0);
   }
 
   /* Close prototype */
@@ -642,20 +690,19 @@ GUILE::functionWrapper(Node *n) {
 
   /* Define the scheme name in C. This define is used by several Guile
      macros. */
-  Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"", 0);
+  Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"\n", 0);
 
   // Now write code to make the function call
   Printv(f->code, tab4, "gh_defer_ints();\n", 0);
-  emit_action(n,f);
+  emit_func_call (name, d, l, f);
   Printv(f->code, tab4, "gh_allow_ints();\n", 0);
 
   // Now have return value, figure out what to do with it.
 
-  if ((tm = Getattr(n,"tmap:out"))) {
-    Replaceall(tm,"$result","gswig_result");
-    Replaceall(tm,"$target","gswig_result");
-    Replaceall(tm,"$source","result");
-    Printv(f->code,tm,"\n",0);
+  if (guile_do_typemap(f->code, "out", d, name,
+		       (char*)"result", (char*)"gswig_result",
+		       0, proc_name, f, 0)) {
+    /* nothing */
   }
   else {
     throw_unhandled_guile_type_error (d);
@@ -669,18 +716,15 @@ GUILE::functionWrapper(Node *n) {
 
   // Look for any remaining cleanup
 
-  if ((NewObject) || (Getattr(n,"feature:new"))) {
-    if ((tm = Swig_typemap_lookup_new("newfree",n,"result",0))) {
-      Replaceall(tm,"$source","result");
-      Printv(f->code,tm,"\n",0);
-    }
+  if (NewObject) {
+    guile_do_typemap(f->code, "newfree", d, iname,
+		     (char*)"result", (char*)"", 0, proc_name, f, 0);
   }
 
   // Free any memory allocated by the function being wrapped..
-  if ((tm = Swig_typemap_lookup_new("ret",n,"result",0))) {
-    Replaceall(tm,"$source","result");
-    Printv(f->code,tm,"\n",0);
-  }
+
+  guile_do_typemap(f->code, "ret", d, name,
+		   (char*)"result", (char*)"", 0, proc_name, f, 0);
 
   // Wrap things up (in a manner of speaking)
 
@@ -703,7 +747,7 @@ GUILE::functionWrapper(Node *n) {
     Printv(f_wrappers, "{\n", 0);
     Printf(f_wrappers, "SCM arg[%d];\n", numargs);
     Printf(f_wrappers, "SWIG_Guile_GetArgs (arg, rest, %d, %d, \"%s\");\n",
-	   numreq, numargs-numreq, proc_name);
+	   numargs-numopt, numopt, proc_name);
     Printv(f_wrappers, "return ", wname, "(", 0);
     Printv(f_wrappers, "arg[0]", 0);
     for (i = 1; i<numargs; i++)
@@ -711,8 +755,8 @@ GUILE::functionWrapper(Node *n) {
     Printv(f_wrappers, ");\n", 0);
     Printv(f_wrappers, "}\n", 0);
     /* Register it */
-    Printf (f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s_rest, 0, 0, 1);\n",
-             proc_name, wname, numreq, numargs-numreq);
+    Printf (f_init, "gh_new_procedure(\"%s\", (SCM (*) ()) %s_rest, 0, 0, 1);\n",
+             proc_name, wname, numargs-numopt, numopt);
   }
   else if (emit_setters && struct_member && strlen(Char(proc_name))>3) {
     int len = Len(proc_name);
@@ -724,8 +768,8 @@ GUILE::functionWrapper(Node *n) {
       struct_member = 2; /* have a setter */
     }
     else Printf(f_init, "SCM getter = ");
-    Printf (f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s, %d, %d, 0);\n",
-	    proc_name, wname, numreq, numargs-numreq);
+    Printf (f_init, "gh_new_procedure(\"%s\", (SCM (*) ()) %s, %d, %d, 0);\n",
+	    proc_name, wname, numargs-numopt, numopt);
     if (!is_setter) {
       /* Strip off "-get" */
       char *pws_name = (char*) malloc(sizeof(char) * (len - 3));
@@ -748,8 +792,8 @@ GUILE::functionWrapper(Node *n) {
   }
   else {
     /* Register the function */
-    Printf (f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s, %d, %d, 0);\n",
-	    proc_name, wname, numreq, numargs-numreq);
+    Printf (f_init, "gh_new_procedure(\"%s\", (SCM (*) ()) %s, %d, %d, 0);\n",
+	    proc_name, wname, numargs-numopt, numopt);
   }
   Printf (exported_symbols, "\"%s\", ", proc_name);
   if (procdoc) {
@@ -769,11 +813,10 @@ GUILE::functionWrapper(Node *n) {
   Delete(returns);
   Delete(tmp);
   DelWrapper(f);
-  return SWIG_OK;
 }
 
 // -----------------------------------------------------------------------
-// GUILE::variableWrapper()
+// GUILE::link_variable(char *name, char *iname, SwigType *d)
 //
 // Create a link to a C variable.
 // This creates a single function PREFIX_var_VARNAME().
@@ -783,84 +826,84 @@ GUILE::functionWrapper(Node *n) {
 // value.
 // -----------------------------------------------------------------------
 
-int
-GUILE::variableWrapper(Node *n)
+void
+GUILE::link_variable (char *name, char *iname, SwigType *t)
 {
-
-  char *name  = GetChar(n,"name");
-  char *iname = GetChar(n,"sym:name");
-  SwigType *t = Getattr(n,"type");
-
   DOHString *proc_name;
   char  var_name[256];
+  char  *tm;
   Wrapper *f;
-  String  *tm;
 
   f = NewWrapper();
   // evaluation function names
 
-  strcpy(var_name, Char(Swig_name_wrapper(iname))); 
+  strcpy(var_name, Char(Swig_name_wrapper(name))); 
 
   // Build the name for scheme.
   proc_name = NewString(iname);
   Replace(proc_name,"_", "-",DOH_REPLACE_ANY);
 
-   if (1 || (SwigType_type(t) != T_USER) || (is_a_pointer(t))) {
+  if ((SwigType_type(t) != T_USER) || (is_a_pointer(t))) {
 
-    Printf (f->def, "static SCM\n%s(SCM s_0)\n{\n", var_name);
+    Printf (f_wrappers, "static SCM %s(SCM s_0)\n{\n", var_name);
 
     /* Define the scheme name in C. This define is used by several Guile
        macros. */
-    Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"", 0);
+    Printv(f_wrappers, "#define FUNC_NAME \"", proc_name, "\"\n", 0);
 
-    Wrapper_add_local (f, "gswig_result", "SCM gswig_result");
-
-    if (!ReadOnly) {
-      /* Check for a setting of the variable value */
-      Printf (f->code, "if (s_0 != GH_NOT_PASSED) {\n");
-      if ((tm = Swig_typemap_lookup_new("varin",n,name,0))) {
-	Replaceall(tm,"$source","s_0");
-	Replaceall(tm,"$input","s_0");
-	Replaceall(tm,"$target",name);
-	Printv(f->code,tm,"\n",0);
-      }
-      else {
-	throw_unhandled_guile_type_error (t);
-      }
-      Printf (f->code, "}\n");
+    if (!(Status & STAT_READONLY) && SwigType_type(t) == T_STRING) {
+      Printf (f_wrappers, "\t char *_temp;\n");
+      Printf (f_wrappers, "\t int  _len;\n");
     }
+    Printf (f_wrappers, "\t SCM gswig_result;\n");
 
-    // Now return the value of the variable (regardless
-    // of evaluating or setting)
+    // Check for a setting of the variable value
 
-    if ((tm = Swig_typemap_lookup_new("varout",n,name,0))) {
-      Replaceall(tm,"$source",name);
-      Replaceall(tm,"$target","gswig_result");
-      Replaceall(tm,"$result", "gswig_result");
-      Printv(f->code,tm,"\n",0);
+    Printf (f_wrappers, "\t if (s_0 != GH_NOT_PASSED) {\n");
+
+    // Yup. Extract the type from s_0 and set variable value
+
+    if (Status & STAT_READONLY) {
+      Printf (f_wrappers, "\t\t scm_misc_error(\"%s\", "
+	       "\"Unable to set %s. Variable is read only.\", SCM_EOL);\n",
+	       proc_name, proc_name);
+    }
+    else if (guile_do_typemap(f_wrappers, "varin",
+			      t, name, (char*) "s_0", name, 1, name, f, 0)) {
+      /* nothing */
     }
     else {
       throw_unhandled_guile_type_error (t);
     }
-    Printf (f->code, "\nreturn gswig_result;\n");
-    Printf (f->code, "#undef FUNC_NAME\n");
-    Printf (f->code, "}\n");
+    Printf (f_wrappers, "\t}\n");
 
-    Wrapper_print (f, f_wrappers);
+    // Now return the value of the variable (regardless
+    // of evaluating or setting)
+
+    if (guile_do_typemap (f_wrappers, "varout",
+			  t, name, name, (char*)"gswig_result",
+			  0, name, f, 1)) {
+      /* nothing */
+    }
+    else {
+      throw_unhandled_guile_type_error (t);
+    }
+    Printf (f_wrappers, "\t return gswig_result;\n");
+    Printf (f_wrappers, "#undef FUNC_NAME\n");
+    Printf (f_wrappers, "}\n\n");
 
     // Now add symbol to the Guile interpreter
 
     if (!emit_setters
-	|| ReadOnly) {
+	|| Status & STAT_READONLY) {
       /* Read-only variables become a simple procedure returning the
-	 value; read-write variables become a simple procedure with
-	 an optional argument. */
-      Printf (f_init, "\t gh_new_procedure(\"%s\", (swig_guile_proc) %s, 0, %d, 0);\n",
-	      proc_name, var_name, ReadOnly ? 0 : 1);
+	 value. */
+      Printf (f_init, "\t gh_new_procedure(\"%s\", (SCM (*) ()) %s, 0, 1, 0);\n",
+	      proc_name, var_name);
     }
     else {
       /* Read/write variables become a procedure with setter. */
-      Printf (f_init, "\t{ SCM p = gh_new_procedure(\"%s\", (swig_guile_proc) %s, 0, 1, 0);\n",
+      Printf (f_init, "\t{ SCM p = gh_new_procedure(\"%s\", (SCM (*) ()) %s, 0, 1, 0);\n",
 	      proc_name, var_name);
       Printf (f_init, "\t  gh_define(\"%s\", "
 	      "scm_make_procedure_with_setter(p, p)); }\n",
@@ -873,41 +916,22 @@ GUILE::variableWrapper(Node *n)
       String *signature = NewString("");
       String *doc = NewString("");
 
-      if (ReadOnly) {
+      if (Status & STAT_READONLY) {
 	Printv(signature, proc_name, 0);
 	Printv(doc, "Returns constant ", 0);
-	if ((tm = Getattr(n,"tmap:varout:doc"))) {
-	  Printv(doc,tm,0);
-	} else {
-	  String *s = SwigType_str(t,0);
-	  Chop(s);
-	  Printf(doc,"<%s>",s);
-	  Delete(s);
-	}
+	guile_do_doc_typemap(doc, "varoutdoc", t, NULL,
+			     0, proc_name, f);
       }
       else {
 	Printv(signature, proc_name,
 	       " #:optional ", 0);
-	if ((tm = Getattr(n,"tmap:varin:doc"))) {
-	  Printv(signature,tm,0);
-	} else {
-	  String *s = SwigType_str(t,0);
-	  Chop(s);
-	  Printf(signature,"new-value <%s>",s);
-	  Delete(s);
-	}
-
+	guile_do_doc_typemap(signature, "varindoc", t, "new-value",
+			     1, proc_name, f);
 	Printv(doc, "If NEW-VALUE is provided, "
 	       "set C variable to this value.\n", 0);
 	Printv(doc, "Returns variable value ", 0);
-	if ((tm = Getattr(n,"tmap:varout:doc"))) {
-	  Printv(doc,tm,0);
-	} else {
-	  String *s = SwigType_str(t,0);
-	  Chop(s);
-	  Printf(doc,"<%s>",s);
-	  Delete(s);
-	}
+	guile_do_doc_typemap(doc, "varoutdoc", t, NULL,
+			     0, proc_name, f);
       }
       write_doc(proc_name, signature, doc);
       Delete(signature);
@@ -921,106 +945,79 @@ GUILE::variableWrapper(Node *n)
   }
   Delete(proc_name);
   DelWrapper(f);
-  return SWIG_OK;
 }
 
 // -----------------------------------------------------------------------
-// GUILE::constantWrapper()
+// GUILE::declare_const(char *name, char *iname, SwigType *type, char *value)
 //
 // We create a read-only variable.
 // ------------------------------------------------------------------------
 
-int
-GUILE::constantWrapper(Node *n)
+void
+GUILE::declare_const (char *name, char *iname, SwigType *type, char *value)
 {
-  char *name      = GetChar(n,"name");
-  char *iname     = GetChar(n,"sym:name");
-  SwigType *type  = Getattr(n,"type");
-  String   *value = Getattr(n,"value");
-
-  int OldReadOnly = ReadOnly;
+  int OldStatus = Status;      // Save old status flags
   DOHString *proc_name;
   char   var_name[256];
   DOHString *rvalue;
+  char   *tm;
   Wrapper *f;
-  SwigType *nctype;
-  String   *tm;
 
   f = NewWrapper();
-  ReadOnly = 1;     // Enable readonly mode.
+  Status = STAT_READONLY;      // Enable readonly mode.
 
   // Make a static variable;
 
-  sprintf (var_name, "%sconst_%s", prefix, iname);
+  sprintf (var_name, "%sconst_%s", prefix, name);
 
-  // Strip const qualifier from type if present
-
-  nctype = NewString(type);
-  if (SwigType_isconst(nctype)) {
-    Delete(SwigType_pop(nctype));
-  }
-  
   // Build the name for scheme.
   proc_name = NewString(iname);
   Replace(proc_name,"_", "-", DOH_REPLACE_ANY);
 
-  if ((SwigType_type(nctype) == T_USER) && (!is_a_pointer(nctype))) {
+  if ((SwigType_type(type) == T_USER) && (!is_a_pointer(type))) {
     Printf (stderr, "%s : Line %d.  Unsupported constant value.\n",
              input_file, line_number);
-    return SWIG_NOWRAP;
+    return;
   }
 
   // See if there's a typemap
 
-  if (SwigType_type(nctype) == T_STRING) {
+  if (SwigType_type(type) == T_STRING) {
     rvalue = NewStringf("\"%s\"", value);
-  } else if (SwigType_type(nctype) == T_CHAR) {
+  } else if (SwigType_type(type) == T_CHAR) {
     rvalue = NewStringf("\'%s\'", value);
   } else {
     rvalue = NewString(value);
   }
-
-  if ((tm = Swig_typemap_lookup_new("constant",n,name,0))) {
-    Replaceall(tm,"$source",rvalue);
-    Replaceall(tm,"$value",rvalue);
-    Replaceall(tm,"$target",name);
-    Printv(f_header,tm,"\n",0);
+  if (guile_do_typemap(f_header, "const", type, name,
+		       Char(rvalue), name, 0, name, f, 0)) {
+    /* nothing */
   } else {
     // Create variable and assign it a value
-    Printf (f_header, "static %s %s = %s;\n", SwigType_lstr(nctype,0),
+    Printf (f_header, "static %s %s = %s;\n", SwigType_lstr(type,0),
 	    var_name, rvalue);
   }
-  {
-    /* Hack alert: will cleanup later -- Dave */
-    Node *n = NewHash();
-    Setattr(n,"name",var_name);
-    Setattr(n,"sym:name",iname);
-    Setattr(n,"type", nctype);
-    variableWrapper(n);
-    Delete(n);
-  }
-  ReadOnly = OldReadOnly;
-  Delete(nctype);
+  // Now create a variable declaration
+  link_variable (var_name, iname, type);
+  Status = OldStatus;
   Delete(proc_name);
   Delete(rvalue);
   DelWrapper(f);
-  return SWIG_OK;
 }
 
-int GUILE::membervariableHandler(Node *n)
+void GUILE::cpp_variable(char *name, char *iname, SwigType *t)
 {
   if (emit_setters) {
     struct_member = 1;
     Printf(f_init, "{\n");
-    Language::membervariableHandler(n);
+    Language::cpp_variable(name, iname, t);
     Printf(f_init, "}\n");
     struct_member = 0;
   }
   else {
     /* Only emit traditional VAR-get and VAR-set procedures */
-    Language::membervariableHandler(n);
+    Language::cpp_variable(name, iname, t);
   }
-  return SWIG_OK;
 }
 
 void GUILE::pragma(char *lang, char *cmd, char *value)
@@ -1029,37 +1026,7 @@ void GUILE::pragma(char *lang, char *cmd, char *value)
     if (strcmp(cmd, (char*)"beforereturn")==0) {
       if (before_return)
 	Delete(before_return);
-      before_return = value ? NewString(value) : NULL;
+      before_return = NewString(value);
     }
   }
 }
-
-
-int GUILE::validIdentifier(String *s) {
-  char *c = Char(s);
-  /* Check whether we have an R5RS identifier.  Guile supports a
-     superset of R5RS identifiers, but it's probably a bad idea to use
-     those. */
-  /* <identifier> --> <initial> <subsequent>* | <peculiar identifier> */
-  /* <initial> --> <letter> | <special initial> */
-  if (!(isalpha(*c) || (*c == '!') || (*c == '$') || (*c == '%')
-	|| (*c == '&') || (*c == '*') || (*c == '/') || (*c == ':')
-	|| (*c == '<') || (*c == '=') || (*c == '>') || (*c == '?')
-	|| (*c == '^') || (*c == '_') || (*c == '~'))) {
-    /* <peculiar identifier> --> + | - | ... */
-    if ((strcmp(c, "+") == 0)
-	|| strcmp(c, "-") == 0
-	|| strcmp(c, "...") == 0) return 1;
-    else return 0;
-  }
-  /* <subsequent> --> <initial> | <digit> | <special subsequent> */
-  while (*c) {
-    if (!(isalnum(*c) || (*c == '!') || (*c == '$') || (*c == '%')
-	  || (*c == '&') || (*c == '*') || (*c == '/') || (*c == ':')
-	  || (*c == '<') || (*c == '=') || (*c == '>') || (*c == '?')
-	  || (*c == '^') || (*c == '_') || (*c == '~') || (*c == '+')
-	  || (*c == '-') || (*c == '.') || (*c == '@'))) return 0;
-    c++;
-  }
-  return 1;
-}
