@@ -15,38 +15,49 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cpp_c[] = "$Header: /cvsroot/swig/SWIG/Source/Preprocessor/cpp.c,v 1.62 2004/11/18 00:25:23 marcelomatus Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Preprocessor/cpp.c,v 1.27.2.1 2001/06/14 08:21:07 mkoeppe Exp $";
 
-#include "swig.h"
 #include "preprocessor.h"
 #include <ctype.h>
 
-static Hash     *cpp = 0;                /* C preprocessor data */
+static DOHHash  *cpp = 0;                /* C preprocessor data */
 static int       include_all = 0;        /* Follow all includes */
-static int       ignore_missing = 0; 
-static int       import_all = 0;         /* Follow all includes, but as %import statements */
-static int       imported_depth = 0;	 /* Depth of %imported files */
 static int       single_include = 1;     /* Only include each file once */
-static Hash     *included_files = 0;
-static List     *dependencies = 0;
+static int       silent_errors = 0;
+static DOHHash  *included_files = 0;
+
+/* Handle an error */
+
+static void cpp_error(DOHString *file, int line, char *fmt, ...) {
+  va_list ap;
+  if (silent_errors) return;
+  va_start(ap,fmt);
+  if (line > 0) {
+    Printf(stderr,"%s:%d ", file, line);
+  } else {
+    Printf(stderr,"%s:EOF ",file);
+  }
+  vPrintf(stderr,fmt,ap);
+  va_end(ap);
+}
 
 /* Test a character to see if it starts an identifier */
 static int
-isidentifier(int c) {
+isidentifier(char c) {
   if ((isalpha(c)) || (c == '_') || (c == '$')) return 1;
   else return 0;
 }
 
 /* Test a character to see if it valid in an identifier (after the first letter) */
 static int
-isidchar(int c) {
+isidchar(char c) {
   if ((isalnum(c)) || (c == '_') || (c == '$')) return 1;
   else return 0;
 }
 
 /* Skip whitespace */
 static void
-skip_whitespace(File *s, File *out) {
+skip_whitespace(DOH *s, DOH *out) {
   int c;
   while ((c = Getc(s)) != EOF) {
     if (!isspace(c)) {
@@ -58,7 +69,7 @@ skip_whitespace(File *s, File *out) {
 
 /* Skip to a specified character taking line breaks into account */
 static int
-skip_tochar(File *s, int ch, File *out) {
+skip_tochar(DOHFile *s, int ch, DOHFile *out) {
   int c;
   while ((c = Getc(s)) != EOF) {
     if (out) Putc(c,out);
@@ -78,43 +89,27 @@ copy_location(DOH *s1, DOH *s2) {
   Setline(s2,Getline(s1));
 }
 
-static String *cpp_include(String_or_char *fn, int sysfile) {
-  String *s;
-  s = sysfile ? Swig_include_sys(fn) : Swig_include(fn);
-  if (s && single_include) {
-    String *file = Getfile(s);
-    if (Getattr(included_files,file)) {
-      Delete(s);
-      return 0;
-    }
-    Setattr(included_files,file,file);
+static DOHString *cpp_include(DOHString_or_char *fn) {
+  DOHString *s;
+  if (single_include) {
+    if (Getattr(included_files,fn)) return 0;
+    Setattr(included_files,fn,fn);
   }
+  s = Swig_include(fn);
   if (!s) {
     Seek(fn,0,SEEK_SET);
-    if (ignore_missing) {
-      Swig_warning(WARN_PP_MISSING_FILE,Getfile(fn),Getline(fn),"Unable to find '%s'\n", fn);
-    } else {
-      Swig_error(Getfile(fn),Getline(fn),"Unable to find '%s'\n", fn);
-    }
+    cpp_error(Getfile(fn),Getline(fn),"Unable to find '%s'\n", fn);
   } else {
     Seek(s,0,SEEK_SET);
-    if (!dependencies) {
-      dependencies = NewList();
-    }
-    Append(dependencies, Copy(Swig_last_file()));
   }
   return s;
 }
 
-List *Preprocessor_depend(void) {
-  return dependencies;
-}
-
 /* -----------------------------------------------------------------------------
  * void Preprocessor_cpp_init() - Initialize the preprocessor
  * ----------------------------------------------------------------------------- */
-void Preprocessor_init(void) {
-  Hash *s;
+void Preprocessor_init() {
+  DOHHash *s;
   cpp = NewHash();
   s =   NewHash();
   Setattr(cpp,"symbols",s);
@@ -128,15 +123,6 @@ void Preprocessor_include_all(int a) {
   include_all = a;
 }
 
-void Preprocessor_import_all(int a) {
-  import_all = a;
-}
-
-void Preprocessor_ignore_missing(int a) {
-  ignore_missing = a;
-}
-
-
 /* -----------------------------------------------------------------------------
  * Preprocessor_define()
  *
@@ -144,56 +130,30 @@ void Preprocessor_ignore_missing(int a) {
  * SWIG macro semantics.
  * ----------------------------------------------------------------------------- */
 
- 
-String_or_char *Macro_vararg_name(String_or_char *str,
-                                  String_or_char *line)
+DOHHash *Preprocessor_define(DOHString_or_char *str, int swigmacro)
 {
-  String_or_char *argname, *varargname;
-  char *s, *dots;
-
-  argname = Copy(str);
-  s = Char(argname);
-  dots = strchr(s, '.');
-  if (!dots) return NULL;
-  if (strcmp(dots, "...") != 0) {
-    Swig_error(Getfile(line), Getline(line),
-               "Illegal macro argument name '%s'\n", str);  
-    return NULL;
-  }
-  if (dots == s) {
-      varargname = NewString("__VA_ARGS__");
-  } else {
-    *dots = '\0';
-    varargname = NewStringf(argname);
-  }
-  Delete(argname);
-  return varargname;
-}
-
-Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
-{
-  String *macroname = 0, *argstr = 0, *macrovalue = 0, *file = 0, *s = 0;
-  Hash   *macro = 0, *symbols = 0, *m1;
-  List   *arglist = 0;
+  DOHString *macroname = 0, *argstr = 0, *macrovalue = 0, *file = 0, *s = 0;
+  DOHHash   *macro = 0, *symbols = 0, *m1;
+  DOHList   *arglist = 0;
   int c, line;
-  int    varargs = 0;
-  String_or_char *str = (String_or_char *) _str;
 
   assert(cpp);
   assert(str);
 
   /* First make sure that string is actually a string */
   if (DohCheck(str)) {
-    s = NewString(str);
+    s = Copy(str);
     copy_location(str,s);
     str = s;
   } else {
     str = NewString((char *) str);
+    Seek(str,0,SEEK_SET);
   }
-  Seek(str,0,SEEK_SET);
   line = Getline(str);
   file = Getfile(str);
 
+  /*     Printf(stdout,"%s:%d '%s'\n", file,line,str); */
+
   /* Skip over any leading whitespace */
   skip_whitespace(str,0);
 
@@ -209,7 +169,7 @@ Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
 	else Putc(c,argstr);
       }
       if (c != ')') {
-	Swig_error(Getfile(str),Getline(str), "Missing \')\' in macro parameters\n");
+	cpp_error(Getfile(str),Getline(str), "Missing \')\' in macro parameters\n");
 	goto macro_error;
       }
       break;
@@ -217,18 +177,9 @@ Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
       Putc(c,macroname);
     } else if (isspace(c)) {
       break;
-    } else if (c == '\\') {
-      c = Getc(str);
-      if (c != '\n') {
-	Ungetc(c,str);
-	Ungetc('\\',str);
-	break;
-      }
     } else {
-      /*Swig_error(Getfile(str),Getline(str),"Illegal character in macro name\n");
-	goto macro_error; */
-      Ungetc(c,str);
-      break;
+      cpp_error(Getfile(str),Getline(str),"Illegal character in macro name\n");
+      goto macro_error;
     }
   }
   if (!swigmacro)
@@ -240,37 +191,24 @@ Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
 
   /* If there are any macro arguments, convert into a list */
   if (argstr) {
-    String *argname, *varargname;
+    DOH *argname;
     arglist = NewList();
     Seek(argstr,0,SEEK_SET);
     argname = NewString("");
     while ((c = Getc(argstr)) != EOF) {
       if (c == ',') {
-	varargname = Macro_vararg_name(argname, str);
-	if (varargname) {
-          Swig_error(Getfile(str),Getline(str),"Variable-length macro argument must be last parameter\n");	  
-	} else {
-	  Append(arglist,argname);
-	}
+	Append(arglist,argname);
 	Delete(argname);
 	argname = NewString("");
-      } else if (isidchar(c) || (c == '.')) {
+      } else if (isidchar(c)) {
 	Putc(c,argname);
-      } else if (!(isspace(c) || (c == '\\'))) {
-	Swig_error(Getfile(str),Getline(str),"Illegal character in macro argument name\n");
+      } else if (!isspace(c)) {
+	cpp_error(Getfile(str),Getline(str),"Illegal character in macro name\n");
 	goto macro_error;
       }
     }
     if (Len(argname)) {
-      /* Check for varargs */
-      varargname = Macro_vararg_name(argname, str);
-      if (varargname) {
-	Append(arglist,varargname);
-	Delete(varargname);
-	varargs = 1;
-      } else {
-	Append(arglist,argname);
-      }
+      Append(arglist,argname);
       Delete(argname);
     }
   }
@@ -278,84 +216,24 @@ Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
   if (!swigmacro) {
     Replace(macrovalue,"\\\n"," ", DOH_REPLACE_NOQUOTE);
   }
-
-  /* Look for special # substitutions.   We only consider # that appears
-     outside of quotes and comments */
-
-  {
-    int state = 0;
-    char *cc = Char(macrovalue);
-    while (*cc) {
-      switch(state) {
-      case 0:
-	if (*cc == '#') *cc = '\001';
-	else if (*cc == '/') state = 10;
-	else if (*cc == '\'') state = 20;
-	else if (*cc == '\"') state = 30;
-	break;
-      case 10:
-	if (*cc == '*') state = 11;
-	else if (*cc == '/') state = 15;
-	else {
-	  state = 0;
-	  cc--;
-	}
-	break;
-      case 11:
-	if (*cc == '*') state = 12;
-	break;
-      case 12:
-	if (*cc == '/') state = 0;
-	else if (*cc != '*') state = 11;
-	break;
-      case 15:
-	if (*cc == '\n') state = 0;
-	break;
-      case 20:
-	if (*cc == '\'') state = 0;
-	if (*cc == '\\') state = 21;
-	break;
-      case 21:
-	state = 20;
-	break;
-      case 30:
-	if (*cc == '\"') state = 0;
-	if (*cc == '\\') state = 31;
-	break;
-      case 31:
-	state = 30;
-	break;
-      default:
-	break;
-      }
-      cc++;
-    }
-  }
-
   /* Get rid of whitespace surrounding # */
-  /*  Replace(macrovalue,"#","\001",DOH_REPLACE_NOQUOTE); */
+  Replace(macrovalue,"#","\001",DOH_REPLACE_NOQUOTE);
   while(strstr(Char(macrovalue),"\001 ")) {
-    Replace(macrovalue,"\001 ","\001", DOH_REPLACE_ANY);
+    Replace(macrovalue,"\001 ","\001", DOH_REPLACE_NOQUOTE);
   }
   while(strstr(Char(macrovalue)," \001")) {
-    Replace(macrovalue," \001","\001", DOH_REPLACE_ANY);
+    Replace(macrovalue," \001","\001", DOH_REPLACE_NOQUOTE);
   }
   /* Replace '##' with a special token */
-  Replace(macrovalue,"\001\001","\002", DOH_REPLACE_ANY);
-  /* Replace '#@' with a special token */
-  Replace(macrovalue,"\001@","\004", DOH_REPLACE_ANY);
-  /* Replace '##@' with a special token */
-  Replace(macrovalue,"\002@","\005", DOH_REPLACE_ANY);  
+  Replace(macrovalue,"\001\001","\002", DOH_REPLACE_NOQUOTE);
 
   /* Go create the macro */
   macro = NewHash();
   Setattr(macro,"name", macroname);
+  Delete(macroname);
   if (arglist) {
     Setattr(macro,"args",arglist);
     Delete(arglist);
-    if (varargs) {
-      Setattr(macro,"varargs","1");
-    }
   }
   Setattr(macro,"value",macrovalue);
   Delete(macrovalue);
@@ -366,23 +244,17 @@ Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
   }
   symbols = Getattr(cpp,"symbols");
   if ((m1 = Getattr(symbols,macroname))) {
-    if (Cmp(Getattr(m1,"value"),macrovalue)) {
-      Swig_error(Getfile(str),Getline(str),"Macro '%s' redefined,\n",macroname);    
-      Swig_error(Getfile(m1),Getline(m1),"previous definition of '%s'.\n",macroname);
-      goto macro_error;
-    }
+    if (Cmp(Getattr(m1,"value"),macrovalue))
+      cpp_error(Getfile(str),Getline(str),"Macro '%s' redefined. Previous definition in \'%s\', Line %d\n", macroname, Getfile(m1), Getline(m1));
   }
   Setattr(symbols,macroname,macro);
-  
   Delete(str);
   Delete(argstr);
-  Delete(macroname);
   return macro;
 
  macro_error:
   Delete(str);
   Delete(argstr);
-  Delete(macroname);
   return 0;
 }
 
@@ -391,9 +263,9 @@ Hash *Preprocessor_define(const String_or_char *_str, int swigmacro)
  *
  * Undefines a macro.
  * ----------------------------------------------------------------------------- */
-void Preprocessor_undef(const String_or_char *str)
+void Preprocessor_undef(DOHString_or_char *str)
 {
-  Hash *symbols;
+  DOH *symbols;
   assert(cpp);
   symbols = Getattr(cpp,"symbols");
   Delattr(symbols,str);
@@ -405,11 +277,11 @@ void Preprocessor_undef(const String_or_char *str)
  * Isolates macro arguments and returns them in a list.   For each argument,
  * leading and trailing whitespace is stripped (ala K&R, pg. 230).
  * ----------------------------------------------------------------------------- */
-static List *
-find_args(String *s)
+static DOHList *
+find_args(DOHString *s)
 {
-  List   *args;
-  String *str;
+  DOHList   *args;
+  DOHString *str;
   int   c, level;
   long  pos;
 
@@ -426,8 +298,7 @@ find_args(String *s)
   if (c != '(') {
     /* Not a macro, bail out now! */
     Seek(s,pos, SEEK_SET);
-    Delete(args);
-    return 0;
+    return args;
   }
   c = Getc(s);
   /* Okay.  This appears to be a macro so we will start isolating arguments */
@@ -473,7 +344,7 @@ find_args(String *s)
     c = Getc(s);
   }
  unterm:
-  Swig_error(Getfile(args),Getline(args),"Unterminated macro call.\n");
+  cpp_error(Getfile(args),Getline(args),"Unterminated macro call.\n");
   return args;
 }
 
@@ -484,63 +355,28 @@ find_args(String *s)
  * or bare.
  * ----------------------------------------------------------------------------- */
 
-static String *
-get_filename(String *str, int* sysfile) {
-  String *fn;
+static DOHString *
+get_filename(DOHString *str) {
+  DOHString *fn;
   int  c;
 
   skip_whitespace(str,0);
   fn = NewString("");
   copy_location(str,fn);
   c = Getc(str);
-  *sysfile = 0;
   if (c == '\"') {
     while (((c = Getc(str)) != EOF) && (c != '\"')) Putc(c,fn);
   } else if (c == '<') {
-    *sysfile = 1;
     while (((c = Getc(str)) != EOF) && (c != '>'))  Putc(c,fn);
   } else {
     Putc(c,fn);
     while (((c = Getc(str)) != EOF) && (!isspace(c))) Putc(c,fn);
     if (isspace(c)) Ungetc(c,str);
   }
-#if defined(_WIN32) || defined(MACSWIG)
-  /* accept Unix path separator on non-Unix systems */
-  Replaceall(fn, "/", SWIG_FILE_DELIMETER);
-#endif
-#if defined(__CYGWIN__)
-  /* accept Windows path separator in addition to Unix path separator */
-  Replaceall(fn, "\\", SWIG_FILE_DELIMETER);
-#endif
   Seek(fn,0,SEEK_SET);
   return fn;
 }
 
-static String *
-get_options(String *str) {
-
-  int  c;
-  skip_whitespace(str,0);
-  c = Getc(str);
-  if (c == '(') {
-    String *opt;
-    int     level = 1;
-    opt = NewString("(");
-    while (((c = Getc(str)) != EOF)) {
-      Putc(c,opt);
-      if (c == ')') {
-	level--;
-	if (!level) return opt;
-      }
-      if (c == '(') level++;
-    }
-    Delete(opt);
-    return 0;
-  } else {
-    Ungetc(c,str);
-    return 0;
-  }
-}
 /* -----------------------------------------------------------------------------
  * expand_macro()
  *
@@ -548,13 +384,14 @@ get_options(String *str) {
  * of error occurred.
  * ----------------------------------------------------------------------------- */
 
-static String *
-expand_macro(String_or_char *name, List *args)
+DOH *expanded_value = 0;
+
+static DOHString *
+expand_macro(DOHString_or_char *name, DOHList *args)
 {
   DOH *symbols, *ns, *macro, *margs, *mvalue, *temp, *tempa, *e;
   DOH *Preprocessor_replace(DOH *);
   int i, l;
-  int isvarargs = 0;
 
   symbols = Getattr(cpp,"symbols");
   if (!symbols) return 0;
@@ -583,45 +420,17 @@ expand_macro(String_or_char *name, List *args)
   assert(mvalue);
   margs = Getattr(macro,"args");
 
-  if (args && Getattr(macro,"varargs")) {
-    isvarargs = 1;
-    /* Variable length argument macro.  We need to collect all of the extra arguments into a single argument */
-    if (Len(args) >= (Len(margs)-1)) {
-      int i;
-      int vi, na;
-      String *vararg = NewString("");
-      vi = Len(margs)-1;
-      na = Len(args);
-      for (i = vi; i < na; i++) {
-	Append(vararg,Getitem(args,i));
-	if ((i+1) < na) {
-	  Append(vararg,",");
-	}
-      }
-      /* Remove arguments */
-      for (i = vi; i < na; i++) {
-	Delitem(args,vi);
-      }
-      Append(args,vararg);
-      Delete(vararg);
-    }
-  }
   /* If there are arguments, see if they match what we were given */
-  if (args && (margs) && (Len(margs) != Len(args))) {
-    if (Len(margs) > (1+isvarargs))
-      Swig_error(Getfile(args),Getline(args),"Macro '%s' expects %d arguments\n", name, Len(margs)-isvarargs);
-    else if (Len(margs) == (1+isvarargs))
-      Swig_error(Getfile(args),Getline(args),"Macro '%s' expects 1 argument\n", name);
+  if ((margs) && (Len(margs) != Len(args))) {
+    if (Len(margs) > 1)
+      cpp_error(Getfile(args),Getline(args),"Macro '%s' expects %d arguments\n", name, Len(margs));
+    else if (Len(margs) == 1)
+      cpp_error(Getfile(args),Getline(args),"Macro '%s' expects 1 argument\n", name);
     else
-      Swig_error(Getfile(args),Getline(args),"Macro '%s' expects no arguments\n", name);
+      cpp_error(Getfile(args),Getline(args),"Macro '%s' expects no arguments\n", name);
     return 0;
   }
 
-  /* If the macro expects arguments, but none were supplied, we leave it in place */
-  if (!args && (margs) && Len(margs) > 0) {
-    return NewString(name);
-  }
-
   /* Copy the macro value */
   ns = Copy(mvalue);
   copy_location(mvalue,ns);
@@ -629,15 +438,19 @@ expand_macro(String_or_char *name, List *args)
   /* Tag the macro as being expanded.   This is to avoid recursion in
      macro expansion */
 
+  if (!expanded_value) {
+    expanded_value = NewString("");
+    DohIntern(expanded_value);
+  }
+  Setattr(macro,"*expanded*",expanded_value);
+
   temp = NewString("");
   tempa = NewString("");
-  if (args && margs) {
+  if (margs) {
     l = Len(margs);
     for (i = 0; i < l; i++) {
       DOH *arg, *aname;
-      String *reparg;
       arg = Getitem(args,i);           /* Get an argument value */
-      reparg = Preprocessor_replace(arg);
       aname = Getitem(margs,i);        /* Get macro argument name */
       if (strstr(Char(ns),"\001")) {
 	/* Try to replace a quoted version of the argument */
@@ -645,20 +458,7 @@ expand_macro(String_or_char *name, List *args)
 	Clear(tempa);
 	Printf(temp,"\001%s", aname);
 	Printf(tempa,"\"%s\"",arg);
-	Replace(ns, temp, tempa, DOH_REPLACE_ID_END);
-      }
-      if (strstr(Char(ns),"\002")) {
-	/* Look for concatenation tokens */
-	Clear(temp);
-	Clear(tempa);
-	Printf(temp,"\002%s",aname);
-	Append(tempa,"\002\003");
-	Replace(ns, temp, tempa, DOH_REPLACE_ID_END);
-	Clear(temp);
-	Clear(tempa);
-	Printf(temp,"%s\002",aname);
-	Append(tempa,"\003\002");
-	Replace(ns,temp,tempa, DOH_REPLACE_ID_BEGIN);
+	Replace(ns, temp, tempa, DOH_REPLACE_ANY);
       }
 
       /* Non-standard macro expansion.   The value `x` is replaced by a quoted
@@ -680,80 +480,27 @@ expand_macro(String_or_char *name, List *args)
 	}
 	Replace(ns,temp,rep, DOH_REPLACE_ANY);
       }
-
-      /* Non-standard mangle expansions.  
-	 The #@Name is replaced by mangle_arg(Name). */
-      if (strstr(Char(ns),"\004")) {
-	String* marg = Swig_string_mangle(arg);
-	Clear(temp);
-	Printf(temp,"\004%s", aname);
-	Replace(ns, temp, marg, DOH_REPLACE_ID_END);
-	Delete(marg);
-      }
-      if (strstr(Char(ns),"\005")) {
-	String* marg = Swig_string_mangle(arg);
-	Clear(temp);
-	Clear(tempa);
-	Printf(temp,"\005%s", aname);
-	Printf(tempa,"\"%s\"", marg);
-	Replace(ns, temp, tempa, DOH_REPLACE_ID_END);
-	Delete(marg);
-      }
-
-      if (isvarargs && i == l-1 && Len(arg) == 0) {
-	/* Zero length varargs macro argument.   We search for commas that might appear before and nuke them */
-	char *a, *s, *t, *name;
-        int namelen;
-	s = Char(ns);
-        name = Char(aname);
-        namelen = Len(aname);
-	a = strstr(s,name);
-	while (a) {
-          if (!isidchar(a[namelen+1])) {
-          /* Matched the entire vararg name, not just a prefix */
-	    t = a-1;
-	    if (*t == '\002') {
-	      t--;
-	      while (t >= s) {
-		if (isspace((int) *t)) t--;
-		else if (*t == ',') {
-		  *t = ' ';
-		} else break;
-	      }
-	    }
-	  }
-	  a = strstr(a+namelen,name);
-	}
-      }
-      /*      Replace(ns, aname, arg, DOH_REPLACE_ID); */
-      Replace(ns, aname, reparg, DOH_REPLACE_ID);   /* Replace expanded args */
-      Replace(ns, "\003", arg, DOH_REPLACE_ANY);    /* Replace unexpanded arg */
-      Delete(reparg);
+      Replace(ns, aname, arg, DOH_REPLACE_ID);
     }
   }
   Replace(ns,"\002","",DOH_REPLACE_ANY);    /* Get rid of concatenation tokens */
   Replace(ns,"\001","#",DOH_REPLACE_ANY);   /* Put # back (non-standard C) */
-  Replace(ns,"\004","#@",DOH_REPLACE_ANY);   /* Put # back (non-standard C) */
+
 
   /* Expand this macro even further */
-  Setattr(macro,"*expanded*","1"); 
-
   e = Preprocessor_replace(ns);
 
-  Delattr(macro,"*expanded*");
   Delete(ns);
-
+  Delattr(macro,"*expanded*");
   if (Getattr(macro,"swigmacro")) {
-    String *g;
-    String *f = NewString("");
+    DOHString *g;
+    DOHString *f = NewString("");
+    Printf(f,"%%macro %s, \"%s\", %d {\n", name, Getfile(macro), Getline(macro));
     Seek(e,0,SEEK_SET);
     copy_location(macro,e);
     g = Preprocessor_parse(e);
-
-    /* Drop the macro in place, but with a marker around it */
-    Printf(f,"/*@%s,%d,%s@*/%s/*@@*/", Getfile(macro), Getline(macro), name, g);
-
-    /*    Printf(f," }\n"); */
+    Printf(f,"%s\n", g);
+    Printf(f,"}\n");
     Delete(g);
     Delete(e);
     e = f;
@@ -763,23 +510,6 @@ expand_macro(String_or_char *name, List *args)
   return e;
 }
 
-/* -----------------------------------------------------------------------------
- * evaluate_args()
- *
- * Evaluate the arguments of a macro 
- * ----------------------------------------------------------------------------- */
-
-List *evaluate_args(List *x) {
-  Iterator i;
-  String *Preprocessor_replace(String *);
-  List *nl = NewList();
-
-  for (i = First(x); i.item; i = Next(i)) {
-    Append(nl,Preprocessor_replace(i.item));
-  }
-  return nl;
-}
-
 /* -----------------------------------------------------------------------------
  * DOH *Preprocessor_replace(DOH *s)
  *
@@ -834,32 +564,29 @@ Preprocessor_replace(DOH *s)
 	Ungetc(c,s);
 	/* See if this is the special "defined" macro */
 	if (Cmp(id,"defined") == 0) {
-	  DOH *args = 0;
+	  DOH *args;
 	  /* See whether or not a paranthesis has been used */
 	  skip_whitespace(s,0);
 	  c = Getc(s);
 	  if (c == '(') {
 	    Seek(s,-1,SEEK_CUR);
 	    args = find_args(s);
-	  } else if (isidchar(c)) {
-	    DOH *arg = NewString("");
+	  } else {
+	    DOH *arg = 0;
 	    args = NewList();
-	    Putc(c,arg);
-	    while (((c = Getc(s)) != EOF)) {
+	    arg = NewString("");
+	    while ((c = Getc(s)) != EOF) {
 	      if (!isidchar(c)) {
 		Seek(s,-1,SEEK_CUR);
 		break;
 	      }
 	      Putc(c,arg);
 	    }
-	    if (Len(arg)) Append(args,arg);
+	    Append(args,arg);
 	    Delete(arg);
-	  } else {
-	    Seek(s,-1,SEEK_CUR);
 	  }
-	  if ((!args) || (!Len(args))) {
-	    /* This is not a defined() macro. */
-	    Printf(ns,"%s",id);
+	  if (!args) {
+	    cpp_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
 	    state = 0;
 	    break;
 	  }
@@ -881,10 +608,7 @@ Preprocessor_replace(DOH *s)
 	  break;
 	}
 	if (Cmp(id,"__FILE__") == 0) {
-	  String *fn = Copy(Getfile(s));
-	  Replaceall(fn,"\\","\\\\");
-	  Printf(ns,"\"%s\"",fn);
-	  Delete(fn);
+	  Printf(ns,"\"%s\"",Getfile(s));
 	  state = 0;
 	  break;
 	}
@@ -896,10 +620,6 @@ Preprocessor_replace(DOH *s)
 	  if (Getattr(m,"args")) {
 	    /* Yep.  We need to go find the arguments and do a substitution */
 	    args = find_args(s);
-	    if (!Len(args)) {
-	      Delete(args);
-	      args = 0;
-	    }
 	  } else {
 	    args = 0;
 	  }
@@ -948,14 +668,14 @@ Preprocessor_replace(DOH *s)
   if (state == 1) {
     /* See if this is the special "defined" macro */
     if (Cmp(id,"defined") == 0) {
-      Swig_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
+      cpp_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
     } else if ((m = Getattr(symbols,id))) {
 	DOH *e;
 	/* Yes.  There is a macro here */
 	/* See if the macro expects arguments */
-	/*	if (Getattr(m,"args")) {
-	  Swig_error(Getfile(id),Getline(id),"Macro arguments expected.\n");
-	  } */
+	if (Getattr(m,"args")) {
+	  cpp_error(Getfile(id),Getline(id),"Macro arguments expected.\n");
+	}
 	e = expand_macro(id,0);
 	Printf(ns,"%s",e);
 	Delete(e);
@@ -978,25 +698,51 @@ Preprocessor_replace(DOH *s)
 static int
 check_id(DOH *s)
 {
-  static SwigScanner *scan = 0;
-  int c;
-  int hastok = 0;
-
+  int c, state = 0;
+  int hasvalue = 0;
   Seek(s,0,SEEK_SET);
+  while ((c = Getc(s)) != EOF) {
+    switch(state) {
 
-  if (!scan) {
-    scan = NewSwigScanner();
+    case 0:
+      if (isdigit(c)) {
+	hasvalue =1;
+	state = 1;
+      }
+      else if (isidentifier(c)) return 1;
+      else if (c == '\"') {
+	skip_tochar(s,'\"',0);
+	hasvalue = 1;
+      } else if (c == '\'') {
+	skip_tochar(s,'\'',0);
+	hasvalue = 1;
+      } else if (c == '/') state = 3;
+      break;
+    case 1:
+      if (isspace(c)) state = 0;
+      hasvalue = 1;
+      break;
+    case 3:
+      if (c == '*') state = 10;
+      else if (c == '/') state = 20;
+      else {
+	Ungetc(c,s);
+	state = 0;
+      }
+      break;
+    case 10:
+      if (c == '*') state = 11;
+      break;
+    case 11:
+      if (c == '/') state = 0;
+      else if (c != '*') state = 10;
+      break;
+    case 20:
+      if (c == '\n') state = 0;
+      break;
+    }
   }
-
-  SwigScanner_clear(scan);
-  s = Copy(s);
-  Seek(s,SEEK_SET,0);
-  SwigScanner_push(scan,s);
-  while ((c = SwigScanner_token(scan))) {
-    hastok = 1;
-    if ((c == SWIG_TOKEN_ID) || (c == SWIG_TOKEN_LBRACE) || (c == SWIG_TOKEN_RBRACE)) return 1;
-  }
-  if (!hastok) return 1;
+  if (!hasvalue) return 1;
   return 0;
 }
 
@@ -1028,28 +774,8 @@ static void add_chunk(DOH *ns, DOH *chunk, int allow) {
   Clear(chunk);
 }
 
-/*
-  push/pop_imported(): helper functions for defining and undefining
-  SWIGIMPORTED (when %importing a file).
- */
-static void
-push_imported() {
-  if (imported_depth == 0) {
-    Preprocessor_define("SWIGIMPORTED 1", 0);
-  }
-  ++imported_depth;
-}
-
-static void
-pop_imported() {
-  --imported_depth;
-  if (imported_depth == 0) {
-    Preprocessor_undef("SWIGIMPORTED");
-  }
-}
-
 /* -----------------------------------------------------------------------------
- * Preprocessor_parse()
+ * DOH *Preprocessor_parse(DOH *s)
  *
  * Parses the string s.  Returns a new string containing the preprocessed version.
  *
@@ -1061,13 +787,12 @@ pop_imported() {
  *           included inline (with all preprocessor directives included).
  * ----------------------------------------------------------------------------- */
 
-String *
-Preprocessor_parse(String *s)
+DOH *
+Preprocessor_parse(DOH *s)
 {
-  String  *ns;             /* New string containing the preprocessed text */
-  String  *chunk, *sval, *decl;
-  Hash    *symbols;
-  String  *id = 0, *value = 0, *comment = 0;
+  DOH   *ns;             /* New string containing the preprocessed text */
+  DOH   *chunk, *symbols, *sval, *decl;
+  DOH   *id = 0, *value = 0, *comment = 0;
   int    i, state, val, e, c;
   int    start_line = 0;
   int    allow = 1;
@@ -1077,9 +802,6 @@ Preprocessor_parse(String *s)
   int    cpp_lines = 0;
   int    cond_lines[256];
 
-  /* Blow away all carriage returns */
-  Replace(s,"\015","",DOH_REPLACE_ANY); 
-
   ns = NewString("");        /* Return result */
 
   decl = NewString("");
@@ -1120,12 +842,12 @@ Preprocessor_parse(String *s)
       else if (c == '\"') {
 	start_line = Getline(s);
 	if (skip_tochar(s,'\"',chunk) < 0) {
-	  Swig_error(Getfile(s),-1,"Unterminated string constant starting at line %d\n",start_line);
+	  cpp_error(Getfile(s),-1,"Unterminated string constant starting at line %d\n",start_line);
 	}
       } else if (c == '\'') {
 	start_line = Getline(s);
 	if (skip_tochar(s,'\'',chunk) < 0) {
-	  Swig_error(Getfile(s),-1,"Unterminated character constant starting at line %d\n",start_line);
+	  cpp_error(Getfile(s),-1,"Unterminated character constant starting at line %d\n",start_line);
 	}
       }
       else if (c == '/') state = 30;  /* Comment */
@@ -1174,13 +896,8 @@ Preprocessor_parse(String *s)
 	if (c == '\n') {
 	  Ungetc(c,s);
 	  state = 50;
-	} else {
-	  state = 42;
-	  if (!isspace(c)) {
-	    Ungetc(c,s);
-	  }
 	}
-
+	else state = 42;
 	copy_location(s,value);
 	break;
       }
@@ -1205,12 +922,6 @@ Preprocessor_parse(String *s)
 	state = 50;
       } else if (c == '/') {
 	state = 45;
-      } else if (c == '\"') {
-	Putc(c,value);
-	skip_tochar(s,'\"',value);
-      } else if (c == '\'') {
-	Putc(c,value);
-	skip_tochar(s,'\'',value);
       } else {
 	Putc(c,value);
 	if (c == '\\') state = 44;
@@ -1218,12 +929,8 @@ Preprocessor_parse(String *s)
       break;
 
     case 44:
-      if (c == '\n') {
-	Putc(c,value);
-	cpp_lines++;
-      } else {
-	Ungetc(c,s);
-      }
+      if (c == '\n') cpp_lines++;
+      Putc(c,value);
       state = 43;
       break;
 
@@ -1275,15 +982,15 @@ Preprocessor_parse(String *s)
 	  if ((m) && !(Getattr(m,"args"))) {
 	    v = Copy(Getattr(m,"value"));
 	    if (Len(v)) {
-	      Swig_error_silent(1);
+	      silent_errors = 1;
 	      v1 = Preprocessor_replace(v);
-	      Swig_error_silent(0);
+	      silent_errors = 0;
 	      /*	      Printf(stdout,"checking '%s'\n", v1); */
 	      if (!check_id(v1)) {
 		if (Len(comment) == 0)
-		  Printf(ns,"%%constant %s = %s;\n", Getattr(m,"name"), v1);
+		  Printf(ns,"%%constant %s %s;\n", Getattr(m,"name"), v1);
 		else
-		  Printf(ns,"%%constant %s = %s; /*%s*/\n", Getattr(m,"name"),v1,comment);
+		  Printf(ns,"%%constant %s %s; /*%s*/\n", Getattr(m,"name"),v1,comment);
 		cpp_lines--;
 	      }
 	      Delete(v1);
@@ -1314,20 +1021,20 @@ Preprocessor_parse(String *s)
 	}
       } else if (Cmp(id,"else") == 0) {
 	if (level <= 0) {
-	  Swig_error(Getfile(s),Getline(id),"Misplaced #else.\n");
+	  cpp_error(Getfile(s),Getline(id),"Misplaced #else.\n");
 	} else {
 	  cond_lines[level-1] = Getline(id);
 	  if (allow) {
 	    allow = 0;
 	    mask = 0;
 	  } else if (level == start_level) {
-	    allow = 1*mask;
+	    allow = 1;
 	  }
 	}
       } else if (Cmp(id,"endif") == 0) {
 	level--;
 	if (level < 0) {
-	  Swig_error(Getfile(id),Getline(id),"Extraneous #endif.\n");
+	  cpp_error(Getfile(id),Getline(id),"Extraneous #endif ignored.\n");
 	  level = 0;
 	} else {
 	  if (level < start_level) {
@@ -1345,11 +1052,8 @@ Preprocessor_parse(String *s)
 	  /*	  Printf(stdout,"Evaluating '%s'\n", sval); */
   	  val = Preprocessor_expr(sval,&e);
   	  if (e) {
-	    char * msg = Preprocessor_expr_error();
   	    Seek(value,0,SEEK_SET);
-	    Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);
-	    if (msg)
-	      Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Error: '%s'\n", msg);
+	    /* cpp_error(Getfile(value),Getline(value),"Could not evaluate '%s'\n", value); */
   	    allow = 0;
   	  } else {
   	    if (val == 0)
@@ -1359,7 +1063,7 @@ Preprocessor_parse(String *s)
   	}
       } else if (Cmp(id,"elif") == 0) {
   	if (level == 0) {
-  	  Swig_error(Getfile(s),Getline(id),"Misplaced #elif.\n");
+  	  cpp_error(Getfile(s),Getline(id),"Misplaced #elif.\n");
   	} else {
   	  cond_lines[level-1] = Getline(id);
   	  if (allow) {
@@ -1370,11 +1074,8 @@ Preprocessor_parse(String *s)
   	    Seek(sval,0,SEEK_SET);
   	    val = Preprocessor_expr(sval,&e);
   	    if (e) {
-	      char * msg = Preprocessor_expr_error();
   	      Seek(value,0,SEEK_SET);
-  	      Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);
-	      if (msg)
-		Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Error: '%s'\n", msg);
+  	      /*      cpp_error(Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);  */
   	      allow = 0;
   	    } else {
   	      if (val)
@@ -1386,57 +1087,24 @@ Preprocessor_parse(String *s)
   	}
       } else if (Cmp(id,"line") == 0) {
       } else if (Cmp(id,"include") == 0) {
-  	if (((include_all) || (import_all)) && (allow)) {
-  	  String *s1, *s2, *fn;
-	  char *dirname; int sysfile = 0;
-	  if (include_all && import_all) {
-	    Swig_warning(WARN_PP_INCLUDEALL_IMPORTALL,Getfile(s),Getline(id),"Both includeall and importall are defined: using includeall");
-	    import_all = 0;
-	  }
+  	if ((include_all) && (allow)) {
+  	  DOH *s1, *s2, *fn;
   	  Seek(value,0,SEEK_SET);
-  	  fn = get_filename(value, &sysfile);
-	  s1 = cpp_include(fn, sysfile);
+  	  fn = get_filename(value);
+	  s1 = cpp_include(fn);
 	  if (s1) {
-	    if (include_all) 
-	      Printf(ns,"%%includefile \"%s\" [\n", Swig_last_file());
-	    else if (import_all) {
-	      Printf(ns,"%%importfile \"%s\" [\n", Swig_last_file());
-	      push_imported();
-	    }
-
-	    /* See if the filename has a directory component */
-	    dirname = Swig_file_dirname(Swig_last_file());
-	    if (sysfile || !strlen(dirname)) dirname = 0;
-	    if (dirname) {
-	      dirname[strlen(dirname)-1] = 0;   /* Kill trailing directory delimeter */
-	      Swig_push_directory(dirname);
-	    }
+  	    Printf(ns,"%%includefile \"%s\" {\n", Swig_last_file());
   	    s2 = Preprocessor_parse(s1);
   	    addline(ns,s2,allow);
-  	    Printf(ns,"\n]");
-	    if (dirname) {
-	      Swig_pop_directory();
-	    }
-	    if (import_all) {
-	      pop_imported();
-	    }
+  	    Printf(ns,"\n}\n");
 	    Delete(s2);
   	  }
 	  Delete(s1);
  	  Delete(fn);
   	}
       } else if (Cmp(id,"pragma") == 0) {
-	if (Strncmp(value,"SWIG ",5) == 0) {
-	  char *c = Char(value)+5;
-	  while (*c && (isspace((int)*c))) c++;
-	  if (*c) {
-	    if (Strncmp(c,"nowarn=",7) == 0) {
-	      Swig_warnfilter(c+7,1);
-	    }
-	  }
-	}
       } else if (Cmp(id,"level") == 0) {
-	Swig_error(Getfile(s),Getline(id),"cpp debug: level = %d, startlevel = %d\n", level, start_level);
+	cpp_error(Getfile(s),Getline(id),"cpp debug: level = %d, startlevel = %d\n", level, start_level);
       }
       for (i = 0; i < cpp_lines; i++)
   	Putc('\n',ns);
@@ -1456,9 +1124,8 @@ Preprocessor_parse(String *s)
       }
       /* %#cpp -  an embedded C preprocessor directive (we strip off the %)  */
       else if (c == '#') {
-	add_chunk(ns,chunk,allow);
   	Putc(c,chunk);
-  	state = 107;
+  	state = 0;
       } else if (isidentifier(c)) {
   	Clear(decl);
   	Putc('%',decl);
@@ -1488,22 +1155,6 @@ Preprocessor_parse(String *s)
   	state = 105;
       }
       break;
-      
-    case 107:
-      Putc(c,chunk);
-      if (c == '\n') {
-	addline(ns,chunk,allow);
-	Clear(chunk);
-	state = 0;
-      } else if (c == '\\') {
-	state = 108;
-      }
-      break;
-
-    case 108:
-      Putc(c,chunk);
-      state = 107;
-      break;
 
     case 110:
       if (!isidchar(c)) {
@@ -1512,39 +1163,22 @@ Preprocessor_parse(String *s)
   	if ((Cmp(decl,"%include") == 0) || (Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
   	  /* Got some kind of file inclusion directive  */
   	  if (allow) {
-  	    DOH *s1, *s2, *fn, *opt; int sysfile = 0;
-
-	    if (Cmp(decl,"%extern") == 0) {
-	      Swig_warning(WARN_DEPRECATED_EXTERN, Getfile(s),Getline(s),"%%extern is deprecated. Use %%import instead.\n");
-	      Clear(decl);
-	      Printf(decl,"%%import");
-	    }
-	    opt = get_options(s);
-  	    fn = get_filename(s, &sysfile);
-	    s1 = cpp_include(fn, sysfile);
+  	    DOH *s1, *s2, *fn;
+  	    fn = get_filename(s);
+	    s1 = cpp_include(fn);
 	    if (s1) {
-	      char *dirname;
   	      add_chunk(ns,chunk,allow);
   	      copy_location(s,chunk);
-  	      Printf(ns,"%sfile%s \"%s\" [\n", decl, opt, Swig_last_file());
-	      if (Cmp(decl,"%import") == 0) {
-		push_imported();
-	      }
-	      dirname = Swig_file_dirname(Swig_last_file());
-	      if (sysfile || !strlen(dirname)) dirname = 0;
-	      if (dirname) {
-		dirname[strlen(dirname)-1] = 0;   /* Kill trailing directory delimeter */
-		Swig_push_directory(dirname);
+  	      Printf(ns,"%sfile \"%s\" {\n", decl, Swig_last_file());
+	      if ((Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
+		Preprocessor_define("WRAPEXTERN 1", 0);
 	      }
   	      s2 = Preprocessor_parse(s1);
-	      if (dirname) {
-		Swig_pop_directory();
-	      }
-	      if (Cmp(decl,"%import") == 0) {
-		pop_imported();
+	      if ((Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
+		Preprocessor_undef("WRAPEXTERN");
 	      }
   	      addline(ns,s2,allow);
-  	      Printf(ns,"\n]");
+  	      Printf(ns,"\n}\n");
 	      Delete(s2);
 	      Delete(s1);
   	    }
@@ -1575,17 +1209,15 @@ Preprocessor_parse(String *s)
       Putc(c,value);
       if (c == '%') {
   	int i = 0;
-  	char *d = "enddef";
-  	for (i = 0; i < 6; i++) {
+  	char *d = "enddef\n";
+  	for (i = 0; i < 7; i++) {
   	  c = Getc(s);
   	  Putc(c,value);
   	  if (c != d[i]) break;
   	}
-	c = Getc(s);
-	Ungetc(c,s);
-  	if ((i == 6) && (isspace(c))) {
+  	if (i == 7) {
   	  /* Got the macro  */
-  	  for (i = 0; i < 7; i++) {
+  	  for (i = 0; i < 8; i++) {
   	    Delitem(value,DOH_END);
   	  }
   	  if (allow) {
@@ -1605,18 +1237,18 @@ Preprocessor_parse(String *s)
     }
   }
   while (level > 0) {
-    Swig_error(Getfile(s),-1,"Missing #endif for conditional starting on line %d\n", cond_lines[level-1]);
+    cpp_error(Getfile(s),-1,"Missing #endif for conditional starting on line %d\n", cond_lines[level-1]);
     level--;
   }
   if (state == 150) {
     Seek(value,0,SEEK_SET);
-    Swig_error(Getfile(s),-1,"Missing %%enddef for macro starting on line %d\n",Getline(value));
+    cpp_error(Getfile(s),-1,"Missing %%enddef for macro starting on line %d\n",Getline(value));
   }
   if ((state >= 105) && (state < 107)) {
-    Swig_error(Getfile(s),-1,"Unterminated %%{ ... %%} block starting on line %d\n", start_line);
+    cpp_error(Getfile(s),-1,"Unterminated %%{ ... %%} block starting on line %d\n", start_line);
   }
   if ((state >= 30) && (state < 40)) {
-    Swig_error(Getfile(s),-1,"Unterminated comment starting on line %d\n", start_line);
+    cpp_error(Getfile(s),-1,"Unterminated comment starting on line %d\n", start_line);
   }
   add_chunk(ns,chunk,allow);
   copy_location(s,chunk);
